#include <stdio.h>
#include <stdlib.h>
#include <math.h>
//------------------------------------------------------------------------------
// Расчет температуры кипения и мольной концентрации этанола в паре при заданных
// давлении и мольной концентрациии этанола в жидкой фазе. Коэффициенты активности 
// рассчитываются в приближении Ван-Лаара (van Laar).
// Для любого компиллятора C стандарта C99 и более поздних. 
// Пример компилляции для gcc из командной строки:
// gcc -std=c99 -o van_Laar van_Laar.c -lm
//------------------------------------------------------------------------------
// 16.12.18  OldBean
//------------------------------------------------------------------------------
// Давление насыщенного пара - по уравнению Антуан
float A1 = 8.21330, B1 = 1652.050, C1 = 231.480; // Этанол
float A2 = 8.07131, B2 = 1730.63, C2 = 233.426; // Вода
float Psat1(float T) {
  return powf(10.0, A1 - B1/(T + C1));
}
float Psat2(float T) {
  return powf(10.0, A2 - B2/(T + C2));
}
//------------------------------------------------------------------------------
// Коэффициенты активности по Ван Лаару (van Laar)
# define M_E 2.7182818284590452354
float A12 = 1.6798, A21 = 0.9227;
float Gam1(float xa) { // Этанол
  float wrk = A21*(1.0 - xa)/(A12*xa + A21*(1.0 - xa));
  return powf(M_E, A12*wrk*wrk);
}
float Gam2(float xa) { // Вода
  float wrk = A12*xa/(A21*(1.0 - xa) + A12*xa);
  return powf(M_E, A21*wrk*wrk);
}
//------------------------------------------------------------------------------
float alpha = 0, beta = 0; // Это xa*Gam1(xa) и (1.0 - xa)*Gam2(xa). 
// Эти глобальные переменные нужны просто для ускорения итерационного процесса 
// по T, где xa уже не изменяется.
float P = 760.0; // Давление
//------------------------------------------------------------------------------
// Модифицированный закон Рауля (с активностями)
// Корень этой функции дает температуру кипения раствора
float Raoult(float T) {
  return Psat1(T)*alpha + Psat2(T)*beta - P; 
}
//------------------------------------------------------------------------------
// Вычисление корня методом дихотомии
float my_root(float aa, float bb, float eps, float its) {
  float a, b;
  if(aa < bb) {a = aa; b = bb;}
  else {a = bb; b = aa;}
  float fa = Raoult(a), fb = Raoult(b);
  if(fa*fb > 0) {
    fprintf(stderr, "В заданном интервале нет корней (или больше одного корня)\n");
    return 0;
  }
  int it = 0; 
  float dx = b - a;
  while(it < its && dx > eps) {
    float c = 0.5*(a + b); 
    float fc = Raoult(c);
    if(fa*fc >= 0) {a = c; fa = fc;}
    else {b = c; fb = fc;}
    dx = b - a; it += 1;
  }
  return 0.5*(a + b);
}
//------------------------------------------------------------------------------
int main(int argc, char **argv) {
  if(argc > 1) P = atof(argv[1]);
  printf("P = %.1f mmHg\n", P);
  printf("x, mole/mole\tT°C\ty, mole/mole\n");
  for(int i = 0; i < 51; i++) {
    float xa = i*0.02; // Мольная концентрация этанола в жидкой фазе
    alpha = xa*Gam1(xa); beta = (1.0 - xa)*Gam2(xa);
    float Tb = my_root(0.0, 150.0, 1.0E-6, 30);
    float ya = Psat1(Tb)*xa*Gam1(xa)/P;
    printf("%.3f\t%.3f\t%.3f\n", xa, Tb, ya);
  }
  return 0;
}
//------------------------------------------------------------------------------
